#include <stdio.h>
#include <string.h>

#define NRBUFFERS 10

typedef char commstr[255];
typedef char msgstr[244];
typedef commstr msgbuf[NRBUFFERS];

extern int system(char *);

class msg {
  int nrBufUsed;
  msgbuf outbuf;
 public:
  msg();
  ~msg();
  void newMsgBuf(int, int, msgstr);
  int addtoMsgBuf(int, int, msgstr);
  int sendOnelineMsg(int, int, msgstr);
  int sendMsgBuf();
  int recvMsgBuf(msgbuf);
  void decipherMsg(commstr, int&, int&, msgstr);
};

/*
 *  Constructor insures msg files empty.
 */
 msg::msg() {
   FILE *fp;
   nrBufUsed = 0;
   if ((fp = fopen("IO.output", "w")) == NULL)
     fprintf(stderr, "Couldn't initialize output message file\n");
   else {
     fprintf(fp, "%d\n", 0);
     fclose(fp);
     system("chmod 666 IO.output");
   }
   if ((fp = fopen("IO.input", "w")) == NULL)
     fprintf(stderr, "Couldn't initialize input message file\n");
   else {
     fprintf(fp, "%d\n", 0);
     fclose(fp);
     system("chmod 666 IO.input");
   }
 }

/*
 *  Destructor empties files before quitting.
 */
 msg::~msg() {
   FILE *fp;
   if ((fp = fopen("IO.output", "w")) != NULL) {
     fprintf(fp, "%d\n", 0);
     fclose(fp);
     system("chmod 666 IO.output");
   }
   if ((fp = fopen("IO.input", "w")) != NULL) {
     fprintf(fp, "%d\n", 0);
     fclose(fp);
     system("chmod 666 IO.input");
   }
 }

/*
 *  Zero message buffer and put one line in it.
 */
void msg::newMsgBuf(int m1, int m2, msgstr msg) {
  sprintf(outbuf[0], "%d %d %s\n", m1, m2, msg);
  nrBufUsed = 1;
}

/*
 *  Add one line to current msg buffer.  Return nr in buffer; 0 if full.
 */
int msg::addtoMsgBuf(int m1, int m2, msgstr msg) {
  if (nrBufUsed == NRBUFFERS) return 0;
  sprintf(outbuf[nrBufUsed++], "%d %d %s\n", m1, m2, msg);
  return nrBufUsed;
}

/*
 *  Send a one line Msg.  Return 1 if sent, 0 if unable to send.
 */
int msg::sendOnelineMsg(int m1, int m2, msgstr msg) {
  FILE *fp;
  int nrInFile;
  if ((fp = fopen("IO.output", "r+")) == NULL) return 0;  /* couldn't open */
  fscanf(fp, "%d ", &nrInFile);
  if (nrInFile > 0) return -1;    /* can't write - not read yet */
  fseek(fp, 0, 0);
  fprintf(fp, "%d\n", 1);
  fprintf(fp, "%d %d %s\n", m1, m2, msg);
  fclose(fp);
  nrBufUsed = 0;
  return 1;
}

/*
 *  Send out whatever is in the msg buffer. Return nr sent if done, 0 if not.
 */
int msg::sendMsgBuf() {
  FILE *fp;
  int i, nrInFile;
  if ((fp = fopen("IO.output", "r+")) == NULL) return 0;  /* couldn't open */
  fscanf(fp, "%d ", &nrInFile);
  if (nrInFile > 0) return -1;    /* can't write - not read yet */
  fseek(fp, 0, 0);
  fprintf(fp, "%d\n", nrBufUsed);
  for (i = 0; i < nrBufUsed; i++)
    fprintf(fp, "%s", outbuf[i]);
  fclose(fp);
  nrBufUsed = 0;
  return i;
}

/*
 *  Read incoming messages.  Return 0 if cannot open file, -1 if file empty,
 *    or nr messages read.
 */
int msg::recvMsgBuf(msgbuf inbuf) {
  FILE *fp;
  int i, nrtoread;
  msgstr msg;
  if ((fp = fopen("IO.input", "r+")) == NULL) return 0;  /* couldn't open */
  fscanf(fp, "%d ", &nrtoread);
  if (nrtoread == 0 || nrtoread > 10) return -1;
  for (i = 0; i < nrtoread; i++) {
    fgets(inbuf[i], sizeof(commstr), fp);
  }
  fseek(fp, 0, 0);
  fprintf(fp, "%d\n", 0);
  fclose(fp);
  return nrtoread;
}

/*
 *  Break msg line up into int,int,str.  Needed because str may have blanks.
 */
void msg::decipherMsg(commstr buf, int& m1, int& m2, msgstr msg) {
  char *ptr;
  sscanf(buf, "%d%d", &m1, &m2);  /* get the 2 int's */
  ptr = strchr(buf, ' ') + 1;   /* point to char after 1st blank */
  ptr = strchr(ptr, ' ') + 1;   /* point to char after 2nd blank */
  strcpy(msg, ptr);             /* copy the remainder to string */
  msg[strlen(msg)-1] = '\0';    /* dump the \n at end */
}
  
main() {          /* test out the IOmgr end of comm's */
  msg msgMgr;
  msgbuf inbuf;
  msgstr mstring;
  int menu1, menu2;
  int i, nrRead, nrtoSend;
  for (;;) {

    /* non-blocking, i.e. don't wait for next message */
    nrRead = msgMgr.recvMsgBuf(inbuf);
    /*------------------------------------------------*/
    
    /* blocking, i.e. don't go on until next message received
       while ((nrRead = msgMgr.revMsgBuf(inbuf)) <= 0);
    ----------------not used here--------------------------*/
    
    if (nrRead <= 0) printf("No messages from IO...\n");
    else {
      for (i = 0; i < nrRead; i++) {
	msgMgr.decipherMsg(inbuf[i], menu1, menu2, mstring);
	printf("Incoming msg %d: %d, %d, %s\n", i+1, menu1, menu2, mstring);
      }
    }
    printf("\nType traffic for IO (RETURN only to send, QUIT to exit)\n");
    nrtoSend = 0;
    printf("Msg %d: ", ++nrtoSend);   /* print prompt */
    gets(mstring);                        /* get user msg */
    if(strlen(mstring) > 0)
      msgMgr.newMsgBuf(0,1, mstring);
    while (strlen(mstring) > 0 && nrtoSend < NRBUFFERS) {
      printf("Msg %d: ", ++nrtoSend);
      gets(mstring);
      if (!strncmp(mstring, "QUIT", 4)) exit(0);
      if(strlen(mstring) > 0) msgMgr.addtoMsgBuf(0, nrtoSend, mstring);
    }
    if (nrtoSend > 0)
      while ((i = msgMgr.sendMsgBuf()) <= 0);  /* keep trying until sent */
  }
}
