#include <setjmp.h>
#include <string.h>
#include <stdio.h>
#include <sys/time.h>
#include <signal.h>
#include "../h/snmperrs.h"
#include "../h/varcvt.h"
#include "snmpmsg.h"  /* should be "../h/snmpmsg.h" eventually */

#define REQ_ANY 0                                  /* any request Id */
#define MIBPREFIX "_iso_org_dod_internet_mgmt_mib_" /* for full var name */
#define PROTOCOL "snmp"                            /* type of msg to send */
#define CMSK    0xff                         /* conversion mask for PhysAddr */
#define PHYSADDR "_mgmt_mib_at_atTable_atEntry_atPhysAddress"

jmp_buf env;                                 /* environment for timeouts */

void onintr() {
  longjmp(env,1);                            /* process timeout interrupt */
}

int errorhandler(errno, errorAction)
     short errno;
     int errorAction;
{
  if (errorAction > 0) {
    fprintf(stderr, "Error no. %d in snmpmsg\n", errno);
    if (errorAction == 2)
      exit(1);
  }
  return errno;
}

void printMsgBody(msg)
     pdu_type msg;
{
  int i, j;
  objident obj;
  printf("Message Body\n");
  printf("------------\n");
  printf("Msg Id Number:    %ld\n", msg.reqid);
  printf("Error status:     %ld\n", msg.errstat);
  printf("Error index:      %ld\n", msg.errindex);
  printf("No. of variables: %d\n", msg.varlist.len);
  for (i = 0; i < msg.varlist.len; i++) {
    printf("Mib var:          ");
    for (j = 0; j < msg.varlist.elem[i].name.ncmp; j++){
      if (j != 0) printf(".");
      printf("%d", msg.varlist.elem[i].name.cmp[j]);
    }
    printf("\n");
  }
}

short formatMsg(mesg,req_id,ac,av)
getreq *mesg;			/* message to be created */
long req_id;			/* request id on packet to be created */
short ac;			/* number of variables to go into list */
objident av[];			/* names of variables to go into list */
{ int i,j;			/* indices and counters */
  if(mesg == NULL || ac <= 0 || av == NULL) /* check for errors */
   return(-1);
/* fill in request id, error indicators */
  mesg->reqid = req_id;		/* request id */
  mesg->errstat = 0;		/* no errors */
  mesg->errindex = 0;		/* meaningless */
  mesg->varlist.len = 0;	/* no variables yet */
/* now fill in the list of variables to be requested */
  for(i = 0; i < ac && i < SNMPMAXVARS; i++)
   {  mesg->varlist.elem[i].name.ncmp = av[i].ncmp; /* length of the name */
      for(j=0; j < av[i].ncmp; j++)
       mesg->varlist.elem[i].name.cmp[j] = av[i].cmp[j];
/* allocate the variable value - dummy value */
      mesg->varlist.elem[i].val.type = EMPTY;  /* type of value */
      (mesg->varlist.len)++;	/* on to next variable */
   }
  if(i > SNMPMAXVARS)
   return(-1);
  return(0);
}

void ConvtoAscii(objPtr, asciiPtr)
     objval *objPtr;
     char   *asciiPtr;
{
  int i;
  char *s;
  u_long t;
  s = asciiPtr + 127;
  *s = '\0';
  switch(objPtr->type) {
  case INT:
    sprintf(asciiPtr, "%d", objPtr->value.intgr);
    break;
  case STR:
    strncpy(asciiPtr, objPtr->value.str.str, 126);
    break;
  case OBJ:
    for (i = 0; i < objPtr->value.obj.ncmp; i++){
      sprintf(asciiPtr, "%u", objPtr->value.obj.cmp[i]);
      asciiPtr += (objPtr->value.obj.cmp[i]) / 10 + 1;
      *(asciiPtr++) = '.';
    }
    break;
  case EMPTY:
    sprintf(asciiPtr, "%s", "NULL");
    break;
  case IPADD:
    sprintf(asciiPtr, "%s", inet_ntoa(objPtr->value.ipadd));
    break;
  case CNTR:
    sprintf(asciiPtr, "%lu", objPtr->value.cntr);
    break;
  case GAUGE:
    sprintf(asciiPtr, "%lu", objPtr->value.gauge);
    break;
  case TIME:
    t = objPtr->value.time;
    sprintf(asciiPtr, "%s", asctime(localtime(&t)));
    asciiPtr[strlen(asciiPtr)-1] = '\0';   /* dump newline char */
    break;
  case OPAQUE:
    sprintf(asciiPtr, "%s", "unprintable (opaque)");
    break;
  default:
    sprintf(asciiPtr, "%s", "unknown value type");
    break;
  }
}

int snmpmsg(msgType, recvIpAddr, communityId, mibVarsPtr, timeout,
            responsePtr, errorAction)
     int msgType;                /* REQ, NXT, etc from snmp.h */
     char *recvIpAddr;           /* receiver in IP format (128.95.1.4) */
     char *communityId;          /* community name */
     vartable *mibVarsPtr;       /* list of MIB var. names (_mgmt_mib...) */
     long timeout;               /* how long to try sending */
     responseList *responsePtr;  /* buffer to hold response */
     int errorAction;            /* 0=return, 1=print/return, 2=print/exit */
{
  short errno;                   /* no. of error found (see snmperrs.h) */
  short nrqueryvars;             /* no. MIB variables to be sent */
  short cIdlen;                  /* length of community name */
  objident queryvarTbl[SNMPMAXVARS]; /* MIB vars in converted msg format */
  strng name;                    /* string with length (see snmp.h) */
  long msgId;                    /* unique no. to identify msg upon return */
  long recvMsgId;                /* Id of msg response recd */
  struct in_addr recvMsgAddr;    /* encoded Ip address to send to */
  struct itimerval interval;     /* timeout interval */
  struct itimerval disable;      /* to disable response timer */
  pdu_type sendmsg;              /* formatted msg (see snmp.h) */
  pdu_type recvmsg;              /* formatted msg (see snmp.h) */
  int i;

  if (mibVarsPtr->nr > SNMPMAXVARS || mibVarsPtr->nr < 1)
    return errorhandler(MIBVARERROR, errorAction);  /* incorrect nr vars */
  for (nrqueryvars = 0; nrqueryvars < mibVarsPtr->nr; nrqueryvars++) {
    name.len = strlen(mibVarsPtr->name[nrqueryvars]) + strlen(MIBPREFIX);
    if ((name.str = (char *)malloc(name.len + 1)) == NULL)
      return errorhandler(EMALLOC, errorAction);
    sprintf(name.str,"%s%s", MIBPREFIX, mibVarsPtr->name[nrqueryvars]);
    name.str[name.len] = '\0';
    if ((errno = symtonum(&name, &queryvarTbl[nrqueryvars])) < 0)
      return errorhandler(errno, errorAction);
  }

  if ((recvMsgAddr.s_addr = inet_addr(recvIpAddr)) == -1)
    errorhandler(NODEST, errorAction);  /* convert dot addr to msg format */

  interval.it_interval.tv_sec = 0;	/* don't want to reload timer */
  interval.it_interval.tv_usec = 0;     /*   upon expiration so set to 0 */
  interval.it_value.tv_sec = timeout;	/* set timeout as expiration */
  interval.it_value.tv_usec = 0;
  disable.it_interval.tv_sec = 0;	/* to disable timer */
  disable.it_interval.tv_usec = 0;
  disable.it_value.tv_sec = 0;
  disable.it_value.tv_usec = 0;

/* create a request message */
  msgId = getpid() ^ time((long *)NULL);	/* XOR process id and time */
  if(formatMsg(&sendmsg, msgId, nrqueryvars, queryvarTbl) < 0)
     return errorhandler(TOOMANYVARS, errorAction);
  cIdlen = strlen(communityId);
  if (setjmp(env))
    return(-46);
  signal(SIGALRM,SIG_IGN);  	    /* ignore timeouts for now */
  if((errno = snmpsend(msgType, msgId,&recvMsgAddr, PROTOCOL, (char *)&sendmsg,
		    communityId, cIdlen, timeout)) < 0)
    return errorhandler(errno, errorAction);
/* prepare to receive */
  recvMsgId = msgId;              
  signal(SIGALRM,onintr);		            /* set for timeout now */
  setitimer(ITIMER_REAL,&interval,(struct itimerval *)NULL); /* SIGALRM on */
  if((errno = snmprecv(&recvMsgAddr, (char *)NULL, &recvMsgId, communityId,
		       &cIdlen, (char *)&recvmsg, sizeof(recvmsg))) < 0)
     return errorhandler(NORECVBUF, errorAction);
  setitimer(ITIMER_REAL,&disable,(struct itimerval *)NULL); /* disable timer */
  if(errno != RSP)			/* if we didn't get a response mesg */
     return errorhandler(NORECVBUF, errorAction);
  if(recvmsg.errstat != 0)		/* if we got an error */
    return recvmsg.errstat;
/*  remove all this to shut the error signals up (returns 1 to 5 if error)
  { 
     fprintf(stderr,"response error: ");
     switch (recvmsg.errstat)
     {
	case 1:
	  fprintf(stderr,"ErrorStatus= tooBig,");
	  break;
	case 2:
	  fprintf(stderr,"ErrorStatus= noSuchName,");
	  break;
	case 3:
	  fprintf(stderr,"ErrorStatus= badValue,");
	  break;
	case 4:
	  fprintf(stderr,"ErrorStatus= readOnly,");
	  break;
	case 5:
	  fprintf(stderr,"ErrorStatus= genErr,");
	  break;
	default:
	  fprintf(stderr,"Unknown Error %d,",recvmsg.errstat);
    }
    fprintf(stderr," ErrorIndex= %d\n",recvmsg.errindex);
    return(-1);
   }
*/
/* just being paranoid here */
  if(recvMsgId != msgId)			/* if got back wrong message */
     return errorhandler(BADRESPONSE, errorAction);
  if(recvmsg.varlist.len < 1)
     return errorhandler(NORESPONSEDATA, errorAction);
/*  free(name.str); name.str = NULL; */
  responsePtr->vars.nr = recvmsg.varlist.len;
  for (i = 0; i < recvmsg.varlist.len; i++) {
    if ((errno = numtosym(&(recvmsg.varlist.elem[i].name), &name)) < 0){
      responsePtr->vars.name[i][0] = '\0';
      responsePtr->val[i].type = -1;
      responsePtr->asciival[i][0] = '\0';
    }
    else {
      bzero(responsePtr->vars.name[i], SNMPSTRLEN);
      bzero(responsePtr->asciival[i], SNMPSTRLEN);
      strcpy(responsePtr->vars.name[i], name.str+31);
      bcopy(&recvmsg.varlist.elem[i].val, &(responsePtr->val[i]),
	    sizeof(recvmsg.varlist.elem[i].val));
      ConvtoAscii(&recvmsg.varlist.elem[i].val, responsePtr->asciival[i]);
    }
  }
  return(0);
}
