#include "listmgr.h"
#include <string.h>
#include <ctype.h>

extern int parseline(char*, char*);
extern int findnetname(nametype, ipaddr);
extern int findnetaddr(nametype, ipaddr);

 listmgr::listmgr() {
   gtwy_listptr = 0;
   gtwy_nractive = 0;
   link_listptr = 0;
   link_nractive = 0;
   LAN_nractive = 0;
   LAN_listptr = 0;
   if (lclwrld.loadworld() < 0)
     fprintf(stderr, "Unable to load LclWorld.data\n");
 }

 listmgr::~listmgr() {
   if (lclwrld.saveworld() < 0)
     fprintf(stderr, "Unable to save LclWorld.data\n");
 }

int listmgr::gtwy_add(ipaddr addr, nametype name, commid cid, int is_op, int s,
		 int t) {
  if (gtwy_nractive == MAXLISTSIZE)
    return -1;
  gmember[gtwy_nractive] = new gateway;
  gmember[gtwy_nractive]->setvars(addr, name, cid, is_op, s, t);
  gtwy_nractive++;
  return 0;
}

gateway* listmgr::gtwy_find(ipaddr addr) {
  int i;
  for (i = 0; i < gtwy_nractive; i++)
    if(strcmp(addr, gmember[i]->getaddr()) == 0)
      return gmember[i];
  return NULLPTR;
}

/*
 *  Returns 0 if removed; -1 if not.  Doesn't delete last gtwy.
 */
int listmgr::gtwy_remove(ipaddr addr) {
  int i;
  if (gtwy_nractive < 2) return -1;
  for (i = 0; i < gtwy_nractive; i++)
    if (strcmp(addr, gmember[i]->getaddr()) == 0)
      break;
  if (i == gtwy_nractive)
    return -1;
  delete gmember[i];
  if (gtwy_listptr >= i)
    gtwy_listptr--;
  for (;i < gtwy_nractive-1; i++)
    gmember[i] = gmember[i+1];
  gtwy_nractive--;
  return 0;
}

gateway* listmgr::gtwy_getfirst() {
  if (gtwy_nractive == 0)
    return NULLPTR;
  gtwy_listptr = 0;
  return gmember[0];
}

gateway* listmgr::gtwy_getnext() {
  if (gtwy_nractive == 0)
    return NULLPTR;
  if (++gtwy_listptr == gtwy_nractive)
    gtwy_listptr = 0;
  return gmember[gtwy_listptr];
}

/*
 * Build the neighbor list for each gateway by looking through the links.
 */
void listmgr::addneighbors() {
  int i, x, y, typ;
  ipaddr a, b;
  gateway *g;
  nametype n;
  for (i = 0; i < link_nractive; i++) {
    lmember[i]->getaddr(a,b);
    if ((g = gtwy_find(a)) != NULLPTR)
      g->addneighbor(b);
    if ((g = gtwy_find(b)) != NULLPTR)
      g->addneighbor(a);
  }
  for (i = 0; i < LAN_nractive; i++) {
    strcpy(n, Lmember[i]->getname());
    Lmember[i]->getfirstmember(a, typ, x, y);
    while (a[0] != '\0') {
      if ((g = gtwy_find(a)) != NULLPTR)
	g->addneighbor(n);
      Lmember[i]->getnextmember(a, typ, x, y);
    }
  }
}

int listmgr::link_add(ipaddr addr1,ipaddr addr2,int x1,int y1,int x2,int y2,
		      int ifTyp, unsigned long ifSpd, int rTyp, int rProto,
		      int intfc1, int intfc2) {
  if (link_nractive == MAXLISTSIZE)
    return -1;
  lmember[link_nractive] = new link;
  if (strcmp(addr1, addr2) < 0) {
    lmember[link_nractive]->setvars(addr1, addr2, x1, y1, x2, y2, ifTyp, ifSpd,
				    rTyp, rProto);
    lmember[link_nractive]->setinterfaces(intfc1, intfc2);
  }
  else {
    lmember[link_nractive]->setvars(addr2, addr1, x2, y2, x1, y1, ifTyp, ifSpd,
				    rTyp, rProto);
    lmember[link_nractive]->setinterfaces(intfc2, intfc1);
  }
  link_nractive++;
  return 0;
}

/*
 *  Search for a link (kept in order of addr1 < addr2).
 */
link* listmgr::link_find(ipaddr ad1, ipaddr ad2) {
  int i;
  ipaddr a, b, tmp, addr1, addr2;
  strcpy(addr1, ad1);
  strcpy(addr2, ad2);
  if (strcmp(addr1, addr2) > 0) {
    strcpy(tmp, addr1);
    strcpy(addr1, addr2);
    strcpy(addr2, tmp);
  }
  for (i = 0; i < link_nractive; i++) {
    lmember[i]->getaddr(a,b);
    if((strcmp(a, addr1) == 0) && (strcmp(b, addr2) == 0))
      return lmember[i];
  }
  return NULLPTR;
}

int listmgr::link_remove(ipaddr ad1, ipaddr ad2) {
  int i;
  ipaddr a, b, tmp, addr1, addr2;
  strcpy(addr1, ad1);
  strcpy(addr2, ad2);
  if (strcmp(addr1, addr2) > 0) {
    strcpy(tmp, addr1);
    strcpy(addr1, addr2);
    strcpy(addr2, tmp);
  }
  for (i = 0; i < link_nractive; i++) {
    lmember[i]->getaddr(a,b);
    if((strcmp(a, addr1) == 0) && (strcmp(b, addr2) == 0))
      break;
  }
  if (i == link_nractive)
    return -1;
  delete lmember[i];
  if (link_listptr >= i)
    link_listptr--;
  for (;i < link_nractive-1; i++)
    lmember[i] = lmember[i+1];
  link_nractive--;
  return 0;
}

link* listmgr::link_getfirst() {
  if (link_nractive == 0)
    return NULLPTR;
  link_listptr = 0;
  return lmember[0];
}

link* listmgr::link_getnext() {
  if (link_nractive == 0)
    return NULLPTR;
  if (++link_listptr == link_nractive)
    link_listptr = 0;
  return lmember[link_listptr];
}

int listmgr::LAN_add(ipaddr name, int iftyp, unsigned long spd, 
		     int x, int y) {
  if (LAN_nractive == MAXLISTSIZE)
    return -1;
  Lmember[LAN_nractive] = new LAN;
  Lmember[LAN_nractive]->setvars(name, iftyp, spd, x, y);
  LAN_nractive++;
  return 0;
}

LAN* listmgr::LAN_find(ipaddr name) {
  int i;
  for (i = 0; i < LAN_nractive; i++)
    if(strcmp(name, Lmember[i]->getname()) == 0)
      return Lmember[i];
  return NULLPTR;
}

int listmgr::LAN_remove(ipaddr name) {
  int i;
  for (i = 0; i < LAN_nractive; i++)
    if (strcmp(name, Lmember[i]->getname()) == 0)
      break;
  if (i == LAN_nractive)
    return -1;
  delete Lmember[i];
  if (LAN_listptr >= i)
    LAN_listptr--;
  for (;i < LAN_nractive-1; i++)
    Lmember[i] = Lmember[i+1];
  LAN_nractive--;
  return 0;
}

LAN* listmgr::LAN_getfirst() {
  if (LAN_nractive == 0)
    return NULLPTR;
  LAN_listptr = 0;
  return Lmember[0];
}

LAN* listmgr::LAN_getnext() {
  if (LAN_nractive == 0)
    return NULLPTR;
  if (++LAN_listptr == LAN_nractive)
    LAN_listptr = 0;
  return Lmember[LAN_listptr];
}

void listmgr::removenetwork() {
  int i;
  for (i = 0; i < gtwy_nractive; i++) 
    delete gmember[i];
  gtwy_nractive = 0;
  gtwy_listptr = 0;
  for (i = 0; i < link_nractive; i++) 
    delete gmember[i];
  link_nractive = 0;
  link_listptr = 0;
  for (i = 0; i < LAN_nractive; i++) 
    delete gmember[i];
  LAN_nractive = 0;
  LAN_listptr = 0;
}

/*
 * Read input file; when 'COMPONENT', send to object to read its own data
 * Return 0: ok-eof;  -1: unexpected eof; -2: list full; -3 gen'l: error
 */
int listmgr::readfile(FILE *fp) {
  char s[128], value[128];
  nametype n;
  int errno = 0, i;
  
  fgets(s, 127, fp);
  for (;;) {
    if (feof(fp)) return -1;                    /* unexpected eof */
    i = parseline(s, value);
    if ((i == 1) && (strncmp(s,"end",3) == 0)) {
      addneighbors();
      return errno;                             /* normal exit */
    }
    if (i != 0) {
      errno = -3;                             /* incorrect line */
      fgets(s, 127, fp);                      /* skip comments/bad data */
    }
    else {
      if (strncmp(s, "comp", 4) == 0) {                 /* a new component */
	if (strncmp(value, "gate", 4) == 0) {           /* a gateway! */
	  if (gtwy_nractive == MAXLISTSIZE) return -2;  /* list full */
	  gmember[gtwy_nractive] = new gateway;         /* create new object */
	  strcpy(s, gmember[gtwy_nractive++]->load(fp));/* gateway read data */
	}                                          /*  rtn next nondata line */
	else {
	  if (strncmp(value, "link", 4) == 0) {           /* a link! */
	    if (link_nractive == MAXLISTSIZE) return -2;  /* list full */
	    lmember[link_nractive] = new link;      /* create new obj */
	    strcpy(s, lmember[link_nractive++]->load(fp));/* link read data */
	  }                                        /*  rtn next nondata line */
	  else {
	    if (strncmp(value, "lan", 4) == 0) {           /* a LAN! */
	      if (LAN_nractive == MAXLISTSIZE) return -2;  /* list full */
	      Lmember[LAN_nractive] = new LAN;             /* create new obj */
	      strcpy(s, Lmember[LAN_nractive++]->load(fp)); 
	    }                                       /* rtn next nondata line */
	    else {
	      errno = -3;                              /* unknown component */
	      fgets(s, 127, fp);
	    }
	  }
	}
      }
      else {
	errno = -3;                                /* unknown input */
	fgets(s, 127, fp);
      }
    }
  }
}

int listmgr::savefile(FILE *fp) {
  int i, j, errno = 0;
  for (i = gtwy_nractive-1; i >= 0; i--) {
    j = gmember[i]->save(fp);
    if (j != 0) errno = j;
  }
  for (i = link_nractive-1; i >= 0; i--) {
    j = lmember[i]->save(fp);
    if (j != 0) errno = j;
  }
  for (i = LAN_nractive-1; i >= 0; i--) {
    j = Lmember[i]->save(fp);
    if (j != 0) errno = j;
  }
  fprintf(fp, "END\n");
  return errno;
}

void listmgr::printgateways() {
  int i, j;
  for (i = 0; i < gtwy_nractive; i++)
    j = gmember[i]->print();
}

void listmgr::printlinks() {
  int i, j;
  for (i = 0; i < link_nractive; i++)
    j = lmember[i]->print();
}

void listmgr::printLANs() {
  int i, j;
  for (i = 0; i < LAN_nractive; i++)
    j = Lmember[i]->print();
}

void listmgr::findcommunity(commid comm, ipaddr addr) {
  gateway *g;
  if ((g = gtwy_find(addr)) != NULLPTR) {
    strcpy(comm, g->getcomm());
    return;
  }
  if (lclwrld.getworldCidfmAddr(comm, addr) > 0) return;
  strcpy(comm, "public");
  return;
}

/*
 *  Locate info about component.  Rtn 0-no info found, 1-in lclworld,
 *  2-in lclworld as alias, 3-found from DNS, 4-foudn as alias from DNS.
 */
int listmgr::findNameAddr(ipaddr addr, ipaddr alias, nametype name, 
			  commid community, int& snmp, int& typ) {
  int i, addrinlclwrld;
  nametype newname;
  if (strcmp(addr, "none") == 0 || strlen(addr) == 0) {  /* user gave name */
    if (strcmp(name, "none") == 0 || strlen(name) == 0) return 0;
    if (lclwrld.getworldAddrfmName(name, addr) == 0)   /* !name in lclwrld */
      if ((i = findnetaddr(name, addr)) != 0)          /* addr not found */
	return 0;
  }
  strcpy(alias, addr);
  if ((addrinlclwrld = lclwrld.addrInworld(addr)) <= 0) {/* not in lwrld */
    if (lclwrld.aliasInworld(alias))                     /* look for alias */
      addrinlclwrld = lclwrld.getworldAddrfmAlias(addr, alias);
    else {
      addrinlclwrld = 0;                             /* not in lclworld */
      if (strcmp(name, "none") == 0 || strlen(name) == 0)
	if ((i = findnetname(newname, addr)) == 0) {    /* if name found */
	  strcpy(name, newname);                       /*   use it */
	}
      if (strcmp(name, "none") == 0 || strlen(name) == 0)/* no supplied name */
	strcpy(name, "unknown");                        /* so name not known */
      if (strcmp(name, "unknown") != 0)         /* if we have a name */
	if ((i = findnetaddr(name, addr)) != 0) /* if nfs addr not found */
	  strcpy(addr, alias);                  /* use value given */
      if (strcmp(addr, alias) != 0)             /* do we have addr fm alias? */
	if ((addrinlclwrld = lclwrld.addrInworld(addr)) <= 0) /* !in lwrld?*/
	  if (lclwrld.aliasInworld(alias))                    /* alias? */
	    addrinlclwrld = lclwrld.getworldAddrfmAlias(addr, alias);
    }
  }
  if (addrinlclwrld > 0) {     /* >0 means lclworld record found */
    i = lclwrld.getworldNamefmAddr(name, addr);
    i = lclwrld.getworldCidfmAddr(community, addr);
    i = lclwrld.getworldSnmpfmAddr(snmp, addr);
    i = lclwrld.getworldTypefmAddr(typ, addr);
    if (strcmp(addr, alias) != 0)
      i = lclwrld.setworldAlias(alias, addr);
  }
  else {
    strcpy(community, "public");
    typ = UNK;
    snmp = UNK;
  } 
  if (addrinlclwrld)
    if (strcmp(alias, addr) == 0) return 1;
    else return 2;
  else
    if (strcmp(alias, addr) == 0) return 3;
    else return 4;
}

/*
 *  Save this component in lclworld. Rtn 0-new, 1-altered existing, 2-no chg.
 */
int listmgr::saveNameAddr(ipaddr addr, ipaddr alias, nametype name,
			  commid community, int snmp, int typ) {
  ipaddr lwaddr;
  char *ptr;
  int i, j, rtn = 2, notsame = 0;
  strcpy(lwaddr, alias);
  strcpy(alias, addr);
  if ((ptr = strchr(lwaddr, '.')) != NULL)
    if ((ptr = strchr(++ptr, '.')) != NULL)
      if ((ptr = strchr(++ptr, '.')) != NULL) {
	ptr++;                                   /* Insure alias is not a */
	j = atoi(ptr);                           /* subnet address */
	if (j != 0) strcpy(alias, lwaddr);
      }
  if (strlen(alias) > 0 && strncmp(addr, alias, strlen(addr) != 0))
    notsame = 1;
  if (lclwrld.addrInworld(addr)) {
    if (notsame && !lclwrld.aliasInworld(alias)) {
      i = lclwrld.setworldAlias(alias, addr);
      rtn = 1;
    }
    if (snmp == YES) {
      i = lclwrld.setworldSnmp(snmp, addr);
      rtn = 1;
    }
    if (typ > 0 && typ < 4) {
      i = lclwrld.setworldType(typ, addr);
      rtn = 1;
    }
    return rtn;
  }
  if (lclwrld.addrInworld(alias)) {
    if (notsame && !lclwrld.aliasInworld(addr)) {
      i = lclwrld.setworldAlias(addr, alias);
      rtn = 1;
    }
    if (typ > 0 && typ < 4) {
      i = lclwrld.setworldType(typ, addr);
      rtn = 1;
    }
    if (snmp == YES)
      i = lclwrld.setworldSnmp(snmp, alias);
    return rtn;
  }
  if (notsame && lclwrld.aliasInworld(addr)) {    /* addr listed as alias? */
    i = lclwrld.getworldAddrfmAlias(lwaddr, addr);
    if (lclwrld.aliasInworld(alias) == 0) {
      i = lclwrld.setworldAlias(alias, lwaddr);
      rtn = 1;
    }
    if (typ > 0 && typ < 4) {
      i = lclwrld.setworldType(typ, addr);
      rtn = 1;
    }
    if (snmp == YES) 
      i = lclwrld.setworldSnmp(snmp, lwaddr);
    return rtn;
  }
  if (notsame && lclwrld.aliasInworld(alias)) {   /* alias listed as alias? */
    i = lclwrld.getworldAddrfmAlias(lwaddr, alias);
    if (lclwrld.aliasInworld(addr) == 0) {
      i = lclwrld.setworldAlias(addr, lwaddr);
      rtn = 1;
    }
    if (typ > 0 && typ < 4) {
      i = lclwrld.setworldType(typ, addr);
      rtn = 1;
    }
    if (snmp == YES) 
      i = lclwrld.setworldSnmp(snmp, lwaddr);
    return rtn;
  }
  rtn = 0;
  i = lclwrld.addtoworld(addr);
  i = lclwrld.setworldCid(community, addr);
  i = lclwrld.setworldSnmp(snmp, addr);
  i = lclwrld.setworldType(typ, addr);
  if (notsame)
    i = lclwrld.setworldAlias(alias, addr);
  return rtn;
}
