#include "../msg/snmpmsg.h"
#include "objects.h"
#include <string.h>
#include <ctype.h>
#include <time.h>

#define LISTSIZE 10

extern int snmpmsg(int, ipaddr, commid, vartable &, long, 
                   responseList &, int);
extern int parseline(char*, char*);
extern long time(long);
extern char* ctime(long*);

 gateway::gateway() {
   strcpy(addr, "0.0.0.0");
   strcpy(name, "unk");
   strcpy(community, "public");
   is_operational = UNK;
   snmp = UNK;
   ctype = UNK;
   xcoord = -1;            /* not drawn */
   ycoord = -1;            /* not drawn */
   neighborlistptr = NULL;
   currneighborptr = NULL;
   monitor_interval = 0;
   last_monitor_time = 0;
 }

void gateway::setvars(ipaddr a, nametype n, commid c, int is_op, int s,
		      int t) {
  strcpy(addr, a);
  strcpy(name, n);
  strcpy(community, c);
  is_operational = is_op;
  snmp = s;
  ctype = t;
}

void gateway::setcoords(int x, int y) {
  xcoord = x;
  ycoord = y;
}

void gateway::addneighbor(ipaddr a) {
  addrlist *newneighbor;
  newneighbor = new addrlist;
  strcpy(newneighbor->addr, a);
  newneighbor->next = neighborlistptr;
  neighborlistptr = newneighbor;
  
}
/*
 *  Remove component from host/gateway list.  Rtn 0 if not found; 1 if okay.
 */
int gateway::removeneighbor(ipaddr a) {
  addrlist *ptr, *priorptr;
  ptr = neighborlistptr;
  if (ptr == NULLPTR) return 0;
  if (strcmp(ptr->addr, a) == 0) {  /* head of list removal */
    if (currneighborptr == neighborlistptr) 
      currneighborptr = ptr->next;
    neighborlistptr = ptr->next;
    delete ptr;
    return 1;
  }
  priorptr = ptr;
  ptr = ptr->next;
  while (ptr != NULLPTR) {
    if (strcmp(ptr->addr, a) == 0) {
      priorptr->next = ptr->next;    /* remove *ptr from list */
      if (currneighborptr == ptr) currneighborptr = ptr->next;
      delete ptr;
      return 1;
    }
    priorptr = ptr;
    ptr = ptr->next;
  }
  return 0;
}

int gateway::getfirstneighbor(ipaddr a) {
  currneighborptr = neighborlistptr;
  if (neighborlistptr == NULLPTR) {
    strcpy(a, NULLSTRING);
    return 0;
  }    
  strcpy(a, neighborlistptr->addr);
  return 1;
}

/*
 * Get next in gateway neighbor list. If empty or end of list return NULL
 */
int gateway::getnextneighbor(ipaddr a) {
  if (currneighborptr == NULLPTR) {
    strcpy(a, NULLSTRING);
    return 0;
  }
  currneighborptr = currneighborptr->next;
  if (currneighborptr == NULLPTR) {
    strcpy(a, NULLSTRING);
    return 0;
  }
  strcpy(a, currneighborptr->addr);
  return 1;
}

int gateway::is_neighbor(ipaddr a) {
  ipaddr adr;
  if (!(getfirstneighbor(adr)))
    return 0;
  while (getnextneighbor(adr)) 
    if (strcmp(adr, a) == 0)
      return 1;
  return 0;
}

int gateway::operational() {
  int errno;
  vartable mibvar;
  responseList response;
  mibvar.nr = 1;
  strcpy(mibvar.name[0], "system_sysDescr_0");
  errno = snmpmsg(REQ, addr, community, mibvar, 10, response, RETURNONERROR);
  if (errno == 0) {
    is_operational = TRUE;
    snmp = YES;
  }
  else {
    is_operational = FALSE;
  }
  return is_operational;
}

int gateway::monitor(int& opstat, nametype data) {
  int errno;
  vartable mibvar;
  responseList response;
  data[0] = '\0';
  opstat = is_operational;
  if (last_monitor_time + monitor_interval > time(NULL))
    return 0;       /* not enough time elapsed since last */
  mibvar.nr = 1;
  strcpy(mibvar.name[0], "system_sysDescr_0");
  errno = snmpmsg(REQ, addr, community, mibvar, 10, response, RETURNONERROR);
  if (errno == 0) {
    is_operational = TRUE;
    snmp = YES;
  }
  else {
    is_operational = FALSE;
  }
  last_monitor_time = time(NULL);
  if (is_operational)  {
    sprintf(data, "%s (%1.33s) monitored at %1.30s", addr, name,
	    ctime(&last_monitor_time));
    opstat = 1;  /* operational */
  }
  else {
    sprintf(data, "%s (%1.33s) not responding at %1.30s", addr, name,
	    ctime(&last_monitor_time));
    opstat = 2;  /* not operational */
  }
  return errno;
}

void gateway::set_monitor(int interval) {
  monitor_interval = interval;
}

char* gateway::load(FILE *fp) {
  char element[128], value[128], s[128];
  static char elist[LISTSIZE][12];
  static int nrelem = 0;
  int i;
  if (nrelem == 0) {
    strcpy(elist[nrelem++], "address");
    strcpy(elist[nrelem++], "name");
    strcpy(elist[nrelem++], "community");
    strcpy(elist[nrelem++], "operational");
    strcpy(elist[nrelem++], "snmp");
    strcpy(elist[nrelem++], "type");
    strcpy(elist[nrelem++], "monitorTime");
    strcpy(elist[nrelem++], "xcoord");
    strcpy(elist[nrelem++], "ycoord");
  }
  for (;;) {
    fgets(element, 127, fp);
    if (feof(fp)) return NULLPTR;         /* unexpected eof */
    strcpy(s, element);
    i = parseline(element, value);
    if (i == 1) return s;                 /* some other valied data? */
    if (i == 0) {                         /* should be valid data */
      if ((strncmp(element,"comp",4) == 0) ||
	  (strncmp(element,"end",3) == 0))
	return s;
      for (i = 0; i < nrelem; i++)
	if (strncmp(element, elist[i], 4) == 0) break;
      switch (i) {
      case 0: strcpy(addr, value);
	      break;
      case 1: strcpy(name, value);
	      break;
      case 2: strcpy(community, value);
	      break;
      case 3: if (strncmp(value, "yes", 3) == 0) is_operational = YES;
	      else
		if (strncmp(value, "no", 2) == 0) is_operational = NO;
	        else is_operational = UNK;
	      break;
      case 4: if (strncmp(value, "yes", 3) == 0) snmp = YES;
	      else
		if (strncmp(value, "no", 2) == 0) snmp = NO;
	        else snmp = UNK;
	      break;
      case 5: if (strncmp(value, "gate", 4) == 0) ctype = GATEWAY;
	      else
		if (strncmp(value, "rout", 4) == 0) ctype = ROUTER;
	        else
		  if (strncmp(value, "host", 4) == 0) ctype = HOST;
	          else ctype = UNK;
	      break;
      case 6: monitor_interval = atol(value);
              if (monitor_interval < 0 || monitor_interval > 86400)
                monitor_interval = 0;
	      break;
      case 7: xcoord = atoi(value);
              if (xcoord < 0 || xcoord > 5000) xcoord = 100;
	      break;
      case 8: ycoord = atoi(value);
              if (ycoord < 0 || ycoord > 5000) ycoord = 100;
	      break;
      default: errno = -1;
	       break;
      }
    }
  }
}

int gateway::save(FILE *fp) {
  char s[32];
  ipaddr a;
  fprintf(fp, "COMPONENT: gateway\n");
  fprintf(fp,"  address:     %s\n", addr);
  fprintf(fp,"  name:        %s\n", name);
  fprintf(fp,"  community:   %s\n", community);
  fprintf(fp,"  xcoord:      %d\n", xcoord);
  fprintf(fp,"  ycoord:      %d\n", ycoord);
  fprintf(fp,"  operational: ");
  switch (is_operational) {
  case UNK: fprintf(fp, "unk\n");
            break;
  case YES: fprintf(fp, "yes\n");
            break;
  case NO:  fprintf(fp, "no\n");
            break;
  default:  fprintf(fp, "unk\n");
            break;
  }
  fprintf(fp,"  snmp:        ");
  switch (snmp) {
  case UNK: fprintf(fp, "unk\n");
            break;
  case YES: fprintf(fp, "yes\n");
            break;
  case NO:  fprintf(fp, "no\n");
            break;
  default:  fprintf(fp, "unk\n");
            break;
  }
  fprintf(fp,"  type:        ");
  switch (ctype) {
  case UNK: fprintf(fp, "unk\n");
            break;
  case GATEWAY: fprintf(fp, "gateway\n");
            break;
  case ROUTER:  fprintf(fp, "router\n");
            break;
  case HOST:  fprintf(fp, "host\n");
            break;
  default:  fprintf(fp, "unk\n");
            break;
  }
  if (monitor_interval > 0) {
    sprintf(s, "%ld", monitor_interval);
    fprintf(fp,"  monitorTime: %s\n", s);
  }
  if (fp == stdout) {
    if (getfirstneighbor(a))
      fprintf(fp,"  neighbors:   %s\n",a);
    else return 0;
    while (getnextneighbor(a))
      fprintf(fp,"               %s\n",a);
  }
  return 0;
}

int gateway::ops_loop(int& opstat, nametype data) {
  int errno = 0, tmperror;
  data[0] = '\0';
  if (monitor_interval > 0) {
    tmperror = monitor(opstat, data);
    if (tmperror != 0 && errno == 0)
      errno = tmperror;
  }
  return errno;
}

 link::link() {
   strcpy(a_addr, "0.0.0.0");
   strcpy(b_addr, "0.0.0.0");
   a_xcoord = -2;            /* -1 = not drawn      */
   a_ycoord = -2;            /* -2 = unknown entity */
   b_xcoord = -2;
   b_ycoord = -2;
   a_interface = 1;
   b_interface = 1;
   intfcType = 1;
   intfcSpeed = 1;
   routeType = 1;
   routeProto = 1;
 }

void link::setvars(ipaddr a, ipaddr b, int ax, int ay, int bx, int by,
		   int ifType, unsigned long ifSpd, int rType, int rProto) {
  strcpy(a_addr, a);
  strcpy(b_addr, b);
  a_xcoord = ax;
  a_ycoord = ay;
  b_xcoord = bx;
  b_ycoord = by;
  intfcType = ifType;
  intfcSpeed = ifSpd;
  routeType = rType;
  routeProto = rProto;
}

void link::getaddr(ipaddr a, ipaddr b) {
  strcpy(a, a_addr);
  strcpy(b, b_addr);
}

void link::geta_coord(int& x, int& y) {
  x = a_xcoord;
  y = a_ycoord;
}

void link::getb_coord(int& x, int& y) {
  x = b_xcoord;
  y = b_ycoord;
}

void link::setcoords(int x1, int y1, int x2, int y2) {
  a_xcoord = x1;
  a_ycoord = y1;
  b_xcoord = x2;
  b_ycoord = y2;
}

void link::setinterfaces(int i1, int i2) {
  a_interface = i1;
  b_interface = i2;
}

char* link::load(FILE *fp) {
  char element[128], value[128], s[128];
  static char elist[LISTSIZE][12];
  static int nrelem = 0;
  int i;
  ipaddr tmp;
  if (nrelem == 0) {
    strcpy(elist[nrelem++], "addr/intfc1");
    strcpy(elist[nrelem++], "addr/intfc2");
    strcpy(elist[nrelem++], "xcoord1");
    strcpy(elist[nrelem++], "ycoord1");
    strcpy(elist[nrelem++], "xcoord2");
    strcpy(elist[nrelem++], "ycoord2");
    strcpy(elist[nrelem++], "intfctype");
    strcpy(elist[nrelem++], "intfcspeed");
    strcpy(elist[nrelem++], "routetype");
    strcpy(elist[nrelem++], "routeproto");
  }
  for (;;) {
    fgets(element, 127, fp);
    if (feof(fp)) return NULLPTR;         /* unexpected eof */
    strcpy(s, element);
    i = parseline(element, value);
    if (i == 1) return s;                 /* some other valid data? */
    if (i == 0) {                         /* should be valid data */
      if ((strncmp(element,"comp",4) == 0) ||
	  (strncmp(element,"end",3) == 0)) {
     /* if (strcmp(a_addr, b_addr) > 0) {  To put link addr in ascending
	  strcpy(tmp, a_addr);               order for better search
          strcpy(a_addr, b_addr);            later, but must switch all
	  strcpy(b_addr, tmp);               link fields or none. This
	}                                    only switches addresses */
	return s;
      }
      for (i = 0; i < nrelem; i++)
	if (strncmp(element, elist[i], 11) == 0) break;
      switch (i) {
      case 0: i = parseline(value, element);
	      strcpy(a_addr, value);
	      if (i == 1)             /* no interface specified */
		a_interface = 1;
	      else {
		a_interface = atoi(element);
		if (a_interface < 0 || a_interface > 100) a_interface = 1;
	      }
	      break;
      case 1: i = parseline(value, element);
	      strcpy(b_addr, value);
	      if (i == 1)             /* no interface specified */
		b_interface = 1;
	      else {
		b_interface = atoi(element);
		if (b_interface < 0 || b_interface > 100) b_interface = 1;
	      }
	      break;
      case 2: a_xcoord = atoi(value);
              if (a_xcoord < 0 || a_xcoord > 5000) a_xcoord = 100;
	      break;
      case 3: a_ycoord = atoi(value);
              if (a_ycoord < 0 || a_ycoord > 5000) a_ycoord = 100;
	      break;
      case 4: b_xcoord = atoi(value);
              if (b_xcoord < 0 || b_xcoord > 5000) b_xcoord = 100;
	      break;
      case 5: b_ycoord = atoi(value);
              if (b_ycoord < 0 || b_ycoord > 5000) b_ycoord = 100;
	      break;
      case 6: intfcType = atoi(value);
              if (intfcType < 0 || intfcType > 22) intfcType = 1;
	      break;
      case 7: intfcSpeed = atol(value);
              if (intfcSpeed < 0) intfcSpeed = 1;
	      break;
      case 8: if (strcmp(value, "invalid") == 0) routeType = INVALID;
	      else
		if (strcmp(value, "direct") == 0) routeType = DIRECT;
	        else
		  if (strcmp(value, "remote") == 0) routeType = REMOTE;
	          else routeType = OTHER;
	       break;
      case 9: routeProto = atoi(value);
              if (routeProto < 0 || routeProto > 13) routeProto = 1;
	      break;
      default: errno = -1;
	       break;
      }
    }
  }
}

int link::save(FILE *fp) {
  fprintf(fp, "COMPONENT: link\n");
  fprintf(fp,"  addr/intfc1: %s: %d\n", a_addr, a_interface);
  fprintf(fp,"  addr/intfc2: %s: %d\n", b_addr, b_interface);
  fprintf(fp,"  xcoord1:     %d\n", a_xcoord);
  fprintf(fp,"  ycoord1:     %d\n", a_ycoord);
  fprintf(fp,"  xcoord2:     %d\n", b_xcoord);
  fprintf(fp,"  ycoord2:     %d\n", b_ycoord);
  fprintf(fp,"  intfcType:   %d\n", intfcType);
  fprintf(fp,"  intfcSpeed:  %lu\n", intfcSpeed);
  switch (routeType) {
  default:;
  case 1:
    fprintf(fp,"  routeType:   other\n");
    break;
  case 2:
    fprintf(fp,"  routeType:   invalid\n");
    break;
  case 3:
    fprintf(fp,"  routeType:   direct\n");
    break;
  case 4:
    fprintf(fp,"  routeType:   remote\n");
    break;
  }
  fprintf(fp,"  routeProto:  %d\n", routeProto);
  return 0;
}

int link::ops_loop(int printflag) {
  int errno = 0;
  printflag = printflag; /* get rid of compile warning */
  return errno;
}

 LAN::LAN() {
   LANlistptr = NULLPTR;
   nextmember = NULLPTR;
   intfcType = 1;        /* other */
   intfcSpeed = 1;
   strcpy(name, NULLSTRING);
   xcoord = 100;
   ycoord = 100;
 }

void LAN::setvars(ipaddr n, int i, unsigned long s, int x, int y) {
  strcpy(name, n);
  intfcType = i;
  intfcSpeed = s;
  xcoord = x;
  ycoord = y;
}

/*
 *  Return 1 if addr is part of LAN; 0 if not.
 */
int LAN::is_member(ipaddr addr) {
  LANaddr *ptr;
  ptr = LANlistptr;
  while (ptr != NULLPTR) {
    if (strcmp(addr, ptr->addr) == 0) return 1;
    ptr = ptr->next;
  }
  return 0;
}

/*
 *  Add new host/gateway to front of LAN list.
 */
void LAN::add_member(ipaddr a, int intfc, int x, int y) {
  LANaddr *ptr;
  ptr = new LANaddr;
  strcpy(ptr->addr,a);
  ptr->interface = intfc;
  ptr->xcoord = x;
  ptr->ycoord = y;
  ptr->next = LANlistptr;
  LANlistptr = ptr;
}

/*
 *  Remove addr (host/gateway) from LAN list. Rtn 0 if not found, 1 if done.
 */
int LAN::del_member(ipaddr a) {
  LANaddr *ptr, *priorptr;
  ptr = LANlistptr;
  if (ptr == NULLPTR) return 0;
  if (strcmp(ptr->addr, a) == 0) {  /* head of list removal */
    if (nextmember == LANlistptr) 
      nextmember = ptr->next;
    LANlistptr = ptr->next;
    delete ptr;
    return 1;
  }
  priorptr = ptr;
  ptr = ptr->next;
  while (ptr != NULLPTR) {
    if (strcmp(ptr->addr, a) == 0) {
      priorptr->next = ptr->next;    /* remove *ptr from list */
      if (nextmember == ptr) nextmember = ptr->next;
      delete ptr;
      return 1;
    }
    priorptr = ptr;
    ptr = ptr->next;
  }
  return 0;
}

/*
 *  Change the xy coordinates for a LAN member.  Return 0 if not found, else 1
 */
int LAN::mv_member(ipaddr a, int x, int y) {
  LANaddr *ptr;
  ptr = LANlistptr;
  while (ptr != NULLPTR) {
    if (strcmp(a, ptr->addr) == 0) {
      ptr->xcoord = x;
      ptr->ycoord = y;
      return 1;
    }
    ptr = ptr->next;
  }
  return 0;
}

/*
 *  Change the interface no. for a LAN member.  Return 0 if not found, else 1
 */
int LAN::setinterface(ipaddr a, int i) {
  LANaddr *ptr;
  ptr = LANlistptr;
  while (ptr != NULLPTR) {
    if (strcmp(a, ptr->addr) == 0) {
      ptr->interface = i;
      return 1;
    }
    ptr = ptr->next;
  }
  return 0;
}

/*
 *  Return the interface number for a LAN member, or 0 if not found.
 */
int LAN::getinterface(ipaddr a) {
  LANaddr *ptr;
  ptr = LANlistptr;
  while (ptr != NULLPTR) {
    if (strcmp(a, ptr->addr) == 0) {
      return ptr->interface;
    }
    ptr = ptr->next;
  }
  return 0;
}

/*
 *  Return the xy coordinates of the LAN symbol.
 */
void LAN::getLANcoord(int& x, int& y) {
  x = xcoord;
  y = ycoord;
}

/*
 *  Set the xy coordinates of the LAN symbol.
 */
void LAN::setLANcoord(int x, int y) {
  xcoord = x;
  ycoord = y;
}


/*
 *  Get first host or gateway on LAN list.  Return 0 if empty, 1 if okay.
 */
int LAN::getfirstmember(ipaddr addr, int& intfc, int& x, int& y) {
  if (LANlistptr == NULLPTR) {
    strcpy(addr, NULLSTRING);
    return 0;
  }
  nextmember = LANlistptr;
  strcpy(addr, nextmember->addr);
  intfc = nextmember->interface;
  x = nextmember->xcoord;
  y = nextmember->ycoord;
  return 1;
}

/*
 *  Get next host/gateway on LAN list. Return 0 if end of list, 1 if okay.
 */
int LAN::getnextmember(ipaddr addr, int& intfc, int& x, int& y) {
  if (nextmember == NULLPTR) {
    strcpy(addr, NULLSTRING);
    return 0;
  }
  nextmember = nextmember->next;
  if (nextmember == NULLPTR) {
    strcpy(addr, NULLSTRING);
    return 0;
  }
  strcpy(addr, nextmember->addr);
  intfc = nextmember->interface;
  x = nextmember->xcoord;
  y = nextmember->ycoord;
  return 1;
}

char* LAN::load(FILE *fp) {
  LANaddr *newLANptr;
  char element[128], value[128], s[128];
  static char elist[LISTSIZE][12];
  static int nrelem = 0;
  int i, interface, x, y;
  if (nrelem == 0) {
    strcpy(elist[nrelem++], "addr/int/x/y");
    strcpy(elist[nrelem++], "intfctype");
    strcpy(elist[nrelem++], "intfcspeed");
    strcpy(elist[nrelem++], "xcoord");
    strcpy(elist[nrelem++], "ycoord");
    strcpy(elist[nrelem++], "subnet");
  }
  for (;;) {
    fgets(element, 127, fp);
    if (feof(fp)) return NULLPTR;         /* unexpected eof */
    strcpy(s, element);
    i = parseline(element, value);
    if (i == 1) return s;                 /* some other valid data? */
    if (i == 0) {                         /* should be valid data */
      if ((strncmp(element,"comp",4) == 0) ||
	  (strncmp(element,"end",3) == 0))
	return s;
      for (i = 0; i < nrelem; i++)
	if (strncmp(element, elist[i], 6) == 0) break;
      switch (i) {
      case 0: 
	i = parseline(value, element);
	newLANptr = new LANaddr;
	strcpy(newLANptr->addr, value);
	newLANptr->interface = 1;
	newLANptr->xcoord = 100;
	newLANptr->ycoord = 100;
	if (i == 1) {    /* no interface or coords, so use defaults */
	  newLANptr->next = LANlistptr;
	  LANlistptr = newLANptr;
	}
	i = parseline(element, value);
	interface = atoi(element);
	if (interface < 0 || interface > 22) interface = 1;
	newLANptr->interface = interface;
	if (i == 1) {  /* no coords, so leave defaults */
	  newLANptr->next = LANlistptr;
	  LANlistptr = newLANptr;
	}
	i = parseline(value, element);
	x = atoi(value);
	if (x < 0 || x > 5000) x = 100;
	newLANptr->xcoord = x;
	if (i == 0) {       /* there was value: element  for x,y */
	  y = atoi(element);
	  if (y < 0 || y > 5000) y = 100;
	  newLANptr->ycoord = y;
	}
	newLANptr->next = LANlistptr;
	LANlistptr = newLANptr;
	break;
      case 1: intfcType = atoi(value);
              if (intfcType < 0 || intfcType > 22) intfcType = 1;
	      break;
      case 2: intfcSpeed = atol(value);
              if (intfcSpeed < 0) intfcSpeed = 1;
	      break;
      case 3: xcoord = atoi(value);
	      if (xcoord < 0 || xcoord > 5000) xcoord = 100;
	      break;
      case 4: ycoord = atoi(value);
	      if (ycoord < 0 || ycoord > 5000) ycoord = 100;
	      break;
      case 5: sprintf(name, "LAN.%s", value);
	      break;
      default: errno = -1;
	       break;
      }
    }
  }
}

int LAN::save(FILE *fp) {
  LANaddr *ptr;
  fprintf(fp, "COMPONENT: LAN\n");
  fprintf(fp,"  subnet:      %s\n", name+4);
  fprintf(fp,"  intfcType:   %d\n", intfcType);
  fprintf(fp,"  intfcSpeed:  %lu\n", intfcSpeed);
  fprintf(fp,"  xcoord:      %d\n", xcoord);
  fprintf(fp,"  ycoord:      %d\n", ycoord);
  ptr = LANlistptr;
  while (ptr != NULLPTR) {
    fprintf(fp, "  addr/int/x/y: %s: %d: %d: %d\n", ptr->addr, ptr->interface,
	    ptr->xcoord, ptr->ycoord);
    ptr = ptr->next;
  }
  return 0;
}

int LAN::ops_loop(int printflag) {
  int errno = 0;
  printflag = printflag; /* get rid of compile warning */
  return errno;
}
