#include <stdio.h>

#define UNK 0
#define YES 1
#define NO  2

#define GATEWAY 1
#define ROUTER  2
#define HOST    3

#define OTHER   1        /* route types */
#define INVALID 2
#define DIRECT  3
#define REMOTE  4

#define TRUE   1
#define FALSE  0

#define NULLPTR 0
#define NULLSTRING "\0"

typedef char ipaddr[32];
typedef char nametype[128];
typedef char commid[16];

struct addrlist {
  ipaddr addr;
  addrlist *next;
};

struct LANaddr {
  ipaddr addr;
  int interface;
  int xcoord;
  int ycoord;
  LANaddr *next;
};

class gateway {
  ipaddr addr;
  nametype name;
  commid community;
  int is_operational;
  int snmp;
  int ctype;
  int xcoord;
  int ycoord;
  addrlist *neighborlistptr;
  addrlist *currneighborptr;
  int monitor_interval;
  long last_monitor_time;
  int monitor(int&, nametype);
 public:
  gateway();
  void setvars(ipaddr, nametype, commid, int, int, int);
  inline ipaddr getaddr()   { return addr; }
  inline nametype getname() { return name; }
  inline commid getcomm()   { return community; }
  inline int getopstat()    { return is_operational; }
  inline int getsnmp()      { return snmp; }
  inline int gettype()      { return ctype; }
  inline int getxcoord()    { return xcoord; }
  inline int getycoord()    { return ycoord; }
  void setcoords(int, int);
  void addneighbor(ipaddr);
  int removeneighbor(ipaddr);
  int getfirstneighbor(ipaddr);
  int getnextneighbor(ipaddr);
  int is_neighbor(ipaddr);
  int operational();
  void set_monitor(int);
  char* load(FILE*);
  int save(FILE*);
  int print()         { return save(stdout); }
  int ops_loop(int&, nametype);
};

class link {
  ipaddr a_addr;
  ipaddr b_addr;
  int a_xcoord;
  int a_ycoord;
  int b_xcoord;
  int b_ycoord;
  int a_interface;
  int b_interface;
  int intfcType;
  unsigned long intfcSpeed;
  int routeType;
  int routeProto;
 public:
  link();
  void setvars(ipaddr, ipaddr, int,int, int,int, int, unsigned long, int, int);
  void getaddr(ipaddr, ipaddr);
  void geta_coord(int&, int&);
  void getb_coord(int&, int&);
  inline int geta_interface()           { return a_interface; }
  inline int getb_interface()           { return b_interface; }
  void setcoords(int,int, int,int);
  void setinterfaces(int, int);
  inline int getintfcType()             { return intfcType; }
  inline unsigned long getintfcSpeed()  { return intfcSpeed; }
  inline int getrouteType()             { return routeType; }
  inline int getrouteProto()            { return routeProto; }
  char* load(FILE*);
  int save(FILE*);
  inline inline int print()             { return save(stdout); }
  int ops_loop(int);
};

class LAN {
  LANaddr *LANlistptr;
  LANaddr *nextmember;
  int intfcType;
  unsigned long intfcSpeed;
  ipaddr name;
  int xcoord;
  int ycoord;
 public:
  LAN();
  void setvars(ipaddr, int, unsigned long, int, int);
  int is_member(ipaddr);
  void add_member(ipaddr, int, int, int);
  int del_member(ipaddr);
  int mv_member(ipaddr, int, int);
  int getinterface(ipaddr);
  int setinterface(ipaddr, int);
  int getfirstmember(ipaddr, int&, int&, int&);
  int getnextmember(ipaddr, int&, int&, int&);  
  void getLANcoord(int&, int&);
  void setLANcoord(int, int);
  inline int getintfcType()            { return intfcType; }
  inline unsigned long getintfcSpeed() { return intfcSpeed; }
  inline ipaddr getname()              { return name; }
  char* load(FILE*);
  int save(FILE*);
  inline int print()                   { return save(stdout); }
  int ops_loop(int);
};
