/* cgenlmenu.c  -  C routine for general menu */

#include <curses.h>
#include <string.h>

#define nrmenus 5

WINDOW* w[5];
static char menulist[5][8];
static int windowsInit = 0;

void cgenlmenu__init()
  {
  }

void cgenlmenu_cInitTerm()
  {
  initscr();
  raw();
  noecho();
  clear();
  }

void cgenlmenu_cQuitTerm()
  {
  clear();
  refresh();
  endwin();
  }

void cgenlmenu_cPrtMsg(s)
  char s[];
  {
  addstr(s);
  refresh();
  }

int cgenlmenu_cOpenNewWindow(windownr, topline, nrwindowlines)
  int windownr, topline, nrwindowlines;
  {
  if (topline + nrwindowlines > LINES - 2) return 1;
  w[windownr] = newwin(nrwindowlines, 0, topline, 0);
  wclear(w[windownr]);
  wrefresh(w[windownr]);
  return 0;
  }

void cgenlmenu_cDrawWindow(windownr, prompt, singlespace, nroptions, 
                           startoption, firstoptionline, optionlist)
  int windownr;
  char prompt[];
  int singlespace, nroptions, startoption, firstoptionline;
  char optionlist[10][80];
  {
  int i, currline, spacing;
  if ((startoption < 1) || (startoption > nroptions))
    startoption = 1;
  if (singlespace) spacing = 1;
  else spacing = 2;
  if (firstoptionline > 0)
    {
    wmove(w[windownr],0,0);
    waddstr(w[windownr],prompt);
    waddch(w[windownr], ':');
    }
  currline = firstoptionline;
  for (i = 1; i <= nroptions; i++)
    {
    wmove(w[windownr], currline, 4);
    if (i == startoption) wstandout(w[windownr]);
    waddstr(w[windownr],optionlist[i-1]);
    wstandend(w[windownr]);
    currline += spacing;
    }
  refresh(w[windownr]);
  }

void Beep()
  {
  printf("%c",7);
  }

void  ScrnInstr()
  {
  move(LINES-1,0);
  standout();
  addch('a');
  standend();
  addstr(" - up,   ");
  standout();
  addch('z');
  standend();
  addstr(" - down,   ");
  standout();
  addstr("<RETURN>");
  standend();
  addstr(" key to select");
  refresh();
  }

void UnHilite(windownr, currentoptnr, currentlinenr, optionlist)
  int windownr, currentoptnr, currentlinenr;
  char optionlist[10][80];
  {
  mvwaddstr(w[windownr], currentlinenr, 4, optionlist[currentoptnr-1]);
  }

void Hilite(windownr, currentoptnr, currentlinenr, optionlist)
  int windownr, currentoptnr, currentlinenr;
  char optionlist[10][80];
  {
  wstandout(w[windownr]);
  mvwaddstr(w[windownr], currentlinenr, 4, optionlist[currentoptnr-1]);
  wstandend(w[windownr]);
  }

int cgenlmenu_cGetUserResponse(windownr, spacing, nroptions,
                                 currentoptnr, firstoptionline, optionlist)
  int windownr, spacing, nroptions, currentoptnr, firstoptionline;
  char optionlist[10][80];
  {    
  char c;
  int currentlinenr;
  if (!spacing) spacing = 2;     /* change from boolean to 1 or 2 lines */ 
  currentlinenr = (currentoptnr - 1) * spacing + firstoptionline;
  ScrnInstr();
  do
    {
    wmove(w[windownr], currentlinenr, 3);
    wrefresh(w[windownr]);
    c = wgetch(w[windownr]);
    switch(c)
      {
      case 'a':;
      case 'A': 
         {
         if (currentoptnr != 1)
           {
           UnHilite(windownr,currentoptnr, currentlinenr,optionlist);
           currentoptnr--;
           currentlinenr -= spacing;
           Hilite(windownr, currentoptnr, currentlinenr, optionlist);
           wrefresh(w[windownr]);
           }
         else Beep();
         break;
         }
      case 'z':;
      case 'Z':
         {
         if (currentoptnr != nroptions)
           {
           UnHilite(windownr,currentoptnr, currentlinenr, optionlist);
           currentoptnr++;
           currentlinenr += spacing;
           Hilite(windownr, currentoptnr, currentlinenr, optionlist);
           wrefresh(w[windownr]);
           }
         else Beep();
         break;
         }
      case 13: return currentoptnr;     /* return key */
      default: Beep();
      }
    }
  while (c != 13);
  }

void cgenlmenu_cCloseWindow(windownr)
  int windownr;
  {
  wclear(w[windownr]);
  wrefresh(w[windownr]);
  delwin(w[windownr]);
  }

void InitTerm() {
  cgenlmenu_cInitTerm();
}

void QuitTerm() {
  cgenlmenu_cQuitTerm();
}

int InMenuList(menuname)
  char menuname[8];
{
  int i;
  for (i = 0; i < nrmenus; i++) 
    if (strcmp(menuname, menulist[i]) == 0) return i;
  return -1;
}

void DeleteFmMenuList(nr)
  int nr;
{
  menulist[nr][0] = '\0';
}

void DeleteMenu(menuname)
  char menuname[8];
{
  int nr;
  nr = InMenuList(menuname);
  if (nr == -1) return;
  else DeleteFmMenuList(nr);
}

static void ComputeLines(singlespace, topline, nrwindowlines, firstoptionline,
                         nroptions, prompt)
  int singlespace, topline, *nrwindowlines, *firstoptionline, nroptions;
  char prompt[80];
{
  if (strlen(prompt) > 0) {
    *nrwindowlines = 2;
    *firstoptionline = 2;
  }
  else {
    *nrwindowlines = 0;
    *firstoptionline = 0;
  }
  if (singlespace) *nrwindowlines += nroptions;
  else *nrwindowlines += nroptions * 2;
}

void BadWindowMsg(nr)
  int nr;
{
  switch (nr) {
  case 1: addstr("Too many windows");
          break;
  case 2: addstr("Window goes below bottom line - 2");
          break;
  default: addstr("General Window Error");
  }
}

int AddToMenuList(menuname)
  char menuname[8];
{
  int nr = 0;
  while (strlen(menulist[nr]) != 0) {
    nr++;
    if (nr > nrmenus - 1) return nr;
  }
  strcpy(menulist[nr], menuname);
  return nr;
}

void InitWindows()
{
  int i;
  for (i = 0; i < nrmenus; i++)
    menulist[i][0] = '\0';
  windowsInit = 1;
}

int GetChoice(menuname, topline, prompt, singlespace, nroptions, 
              startoption, optionlist, leaveonscreen)
  char menuname[8];
  int topline;
  char prompt[80];
  int singlespace;
  int nroptions;
  int startoption;
  char optionlist[10][80];
  int leaveonscreen;
{
  int nrwindowlines, firstoptionline, userchoice, windownr;
  int hasprompt, windowtoobig, toomanywindows;
  if (!windowsInit) InitWindows();
  ComputeLines(singlespace, topline, &nrwindowlines, &firstoptionline,
               nroptions, prompt);
  windownr = InMenuList(menuname);
  if (windownr == -1) {
    windownr = AddToMenuList(menuname);
    if (windownr > nrmenus -1) {
      BadWindowMsg(1);
      return 0;
    }
    windowtoobig = cgenlmenu_cOpenNewWindow(windownr, topline, nrwindowlines);
    if (windowtoobig) {
      BadWindowMsg(2);
      cgenlmenu_cCloseWindow(windownr);
      DeleteFmMenuList(windownr);
      return 0;
    }
    cgenlmenu_cDrawWindow(windownr, prompt, singlespace, nroptions,
                          startoption, firstoptionline, optionlist);
  }
  userchoice = cgenlmenu_cGetUserResponse(windownr, singlespace, nroptions,
                               startoption, firstoptionline, optionlist);
  if (!leaveonscreen) {
    cgenlmenu_cCloseWindow(windownr);
    DeleteFmMenuList(windownr);
  }
  return userchoice;
}
