#include "entry.h"
#include <stdio.h>
#include "tio.h"
#include <string.h>
#include <time.h>

/* #define MSGCHECK 0 */

extern long time(long);
extern int GetChoice(mname, int, mstring, int, int, int, optlist, int);
extern void InitTerm();
extern void QuitTerm();
extern void DeleteMenu(mname);
extern int query(mstring, long&, double&, entryForm, entryType, int, int,
		 int, int);
extern int system(char*);

/*
 *  Constructor insures msg files empty.
 */
 msg::msg() {
   FILE *fp;
   nrBufUsed = 0;
   if ((fp = fopen("IO.output", "w")) == NULL)
     fprintf(stderr, "Couldn't initialize output message file\n");
   else {
     fprintf(fp, "%d\n", 0);
     fclose(fp);
   }
   if ((fp = fopen("IO.input", "w")) == NULL)
     fprintf(stderr, "Couldn't initialize input message file\n");
   else {
     fprintf(fp, "%d\n", 0);
     fclose(fp);
   }
 }

/*
 *  Zero message buffer and put one line in it.
 */
void msg::newMsgBuf(int m1, int m2, msgstr msg) {
  sprintf(inbuf[0], "%d %d %s\n", m1, m2, msg);
  nrBufUsed = 1;
}

/*
 *  Add one line to current msg buffer.  Return nr in buffer; 0 if full.
 */
int msg::addtoMsgBuf(int m1, int m2, msgstr msg) {
  if (nrBufUsed == NRBUFFERS) return 0;
  sprintf(inbuf[nrBufUsed++], "%d %d %s\n", m1, m2, msg);
  return nrBufUsed;
}

/*
 *  Send a one line Msg.  Return 1 if sent, 0 if unable to send.
 */
int msg::sendOnelineMsg(int m1, int m2, msgstr msg) {
  FILE *fp;
  int nrInFile;
  if ((fp = fopen("IO.input", "r+")) == NULL) return 0;
  fscanf(fp, "%d ", &nrInFile);
  if (nrInFile > 0) {
    fclose(fp);
    return -1;
  }
  fseek(fp, 0, 0);
  fprintf(fp, "%d\n", 1);
  fprintf(fp, "%d %d %s\n", m1, m2, msg);
  fclose(fp);
  nrBufUsed = 0;
#ifdef MSGCHECK
  printf("tio sent: %d %d %s\n", m1, m2, msg);
#endif
  return 1;
}

/*
 *  Send out whatever is in the msg buffer. Return nr sent if done, 0 if not.
 */
int msg::sendMsgBuf() {
  FILE *fp;
  int i, nrInFile;
  if ((fp = fopen("IO.input", "r+")) == NULL) return 0;  /* couldn't open */
  fscanf(fp, "%d ", &nrInFile);
  if (nrInFile > 0) {
    fclose(fp);
    return -1;    /* can't write - not read yet */
  }
  fseek(fp, 0, 0);
  fprintf(fp, "%d\n", nrBufUsed);
  for (i = 0; i < nrBufUsed; i++)
    fprintf(fp, "%s", inbuf[i]);
  fclose(fp);
  nrBufUsed = 0;
  return i;
}

/*
 *  Read incoming messages.  Return 0 if cannot open file, -1 if file empty,
 *    or nr messages read.
 */
int msg::recvMsgBuf(msgbuf outbuf) {
  FILE *fp;
  int i, nrtoread;
  msgstr msg;
  if ((fp = fopen("IO.output", "r+")) == NULL) {
    return 0;  /* couldn't open */
  }
  fscanf(fp, "%d ", &nrtoread);
  if (nrtoread == 0 || nrtoread > 10) {
    fclose(fp);
    return -1;
  }
  for (i = 0; i < nrtoread; i++) {
    fgets(outbuf[i], sizeof(commstr), fp);
#ifdef MSGCHECK
    printf("tio recd: %s", outbuf[i]);
#endif
  }
  fseek(fp, 0, 0);
  fprintf(fp, "%d\n", 0);
  fclose(fp);
  return nrtoread;
}

/*
 *  Break msg line up into int,int,str.  Needed because str may have blanks.
 */
void msg::decipherMsg(commstr buf, int& m1, int& m2, msgstr msg) {
  char *ptr;
  sscanf(buf, "%d%d", &m1, &m2);  /* get the 2 int's */
  ptr = strchr(buf, ' ') + 1;   /* point to char after 1st blank */
  ptr = strchr(ptr, ' ') + 1;   /* point to char after 2nd blank */
  strcpy(msg, ptr);             /* copy the remainder to string */
  msg[strlen(msg)-1] = '\0';    /* dump the \n at end */
}

 menu::menu() {
   lastoption = 1;
   nroptions = 0;
 }

void menu::deletemenu(mname n) {
  DeleteMenu(n);
}

void menu::setup(mname nm, int tp, mstring pr, int nropt, optlist opt) {
  strcpy(name, nm);
  topline = tp;
  strcpy(prompt, pr);
  nroptions = nropt;
  for (int i = 0; i < nropt; i++)
    strcpy(optionlist[i], opt[i]);
}

/*
 * nextopt: 0=use last; 1=start @ 1   leaveonscreen: 0=no; 1=yes
 */
int menu::getresponse(int nextopt, int leaveonscreen) {
  if (nextopt == 0)
    nextopt = lastoption;
  else
    nextopt = 1;
  lastoption = GetChoice(name, topline, prompt, 1, nroptions, nextopt,
			 optionlist, leaveonscreen);
  return lastoption;
}

 iomgr::iomgr() {
  optlist optionlist;
  mstring prompt;
  strcpy(prompt, "Select next option");
  strcpy(optionlist[0], "Load a saved network file");
  strcpy(optionlist[1], "Save current network");
  strcpy(optionlist[2], "Print a network list");
  strcpy(optionlist[3], "Turn log file on/off");
  strcpy(optionlist[4], "Add a new component");
  strcpy(optionlist[5], "Delete a component");
  strcpy(optionlist[6], "Go to functions menu");
  strcpy(optionlist[7], "Terminate the program");
  menu1.setup("menu1", 0, prompt, 8, optionlist);
  strcpy(prompt, "Select function");
  strcpy(optionlist[0], "Query an MIB variable");
  strcpy(optionlist[1], "Query an MIB category");
  strcpy(optionlist[2], "Turn component monitor on/off");
  strcpy(optionlist[3], "Check operational status of a component");
  strcpy(optionlist[4], "Analyze component performance");
  strcpy(optionlist[5], "Change network map");
  strcpy(optionlist[6], "Trace a route");
  strcpy(optionlist[7], "Cancel this menu");
  menu2.setup("menu2", 0, prompt, 8, optionlist);
  strcpy(prompt, "Select type of list:");
  strcpy(optionlist[0], "Network node summary");
  strcpy(optionlist[1], "Network link summary");
  strcpy(optionlist[2], "Network LAN summary");
  strcpy(optionlist[3], "Cancel this menu");
  menu3.setup("menu3", 12, prompt, 4, optionlist);
  nextToRead = nrOutWaiting = dialogBoxOpen = 0;
  logfileon = 0;
}

/*
 * Switch log file on/off; Return 0 if not changed; 1 if switched.
 */
int iomgr::logfileOnOff(msgstr inputstr) {
  int escpress;
  long l;
  double d;
  gotoxy(0,12);
  if (logfileon) {
    addstr("Log file is on - Turn off (Y/N)?");
    escpress = query(inputstr, l, d, NEW, YN, 33, 12, 1, 1);
    if (l == 1) {
      logfileon = 0;
      inputstr[0] = '\0';
      return 1;
    }
    else return 0;
  }
  else {
    addstr("Log file is off - Turn on (Y/N)?");
    escpress = query(inputstr, l, d, NEW, YN, 33, 12, 1, 1);
    if (l == 1) {
      gotoxy(0,14);
      addstr("Enter new log file name:");
      escpress = query(inputstr, l, d, ESCNEW, STR, 25, 14, 0, 30);
      if (escpress || strlen(inputstr) == 0)
	return 0;
      else {
	logfileon = 1;
        return 1;
      }
    }
    else return 0;
  }
}

int iomgr::getmenu2choice(msgstr inputdata) {
  int m2choice, escpress;
  long l;
  double d;
  char c;
  mstring data1, data2, data3;
  clear();
  m2choice = menu2.getresponse(0,1);
  gotoxy(0,12);
  switch(m2choice) {
  case 3:
    addstr("Enter IP address of component to monitor (ESC to cancel):");
    escpress = query(data1, l, d, ESCNEW, STR, 58, 12, 0, 16);
    if (escpress || strlen(data1) == 0)
      m2choice = 0;
    else {
      gotoxy(0,13);
      addstr("Enter monitor interval (in sec):");
      escpress = query(data2, l, d, NEW, LINT, 33, 13, 0, 6);
      sprintf(inputdata, "%s %s %ld", data1, "none", l);
    }
    break;
  case 4:
    addstr("Enter IP address of component (ESC to cancel):");
    escpress = query(inputdata, l, d, ESCNEW, STR, 47, 12, 0, 16);
    if (escpress || strlen(inputdata) == 0)
      m2choice = 0;
    else strcat(inputdata, " none");
    break;
  case 8:
    m2choice = 0;
    break;
  case 1:;
  case 2:
    addstr("Enter IP address of component (ESC to cancel):");
    escpress = query(data1, l, d, ESCNEW, STR, 47, 12, 0, 16);
    if (escpress || strlen(data1) == 0) {
      m2choice = 0;
      break;
    }
    gotoxy(0,14);
    addstr("MIB variable:");
    escpress = query(data2, l, d, ESCNEW, STR, 14, 14, 0, 60);
    if (escpress || strlen(data2) == 0) {
      m2choice = 0;
      break;
    }
    gotoxy(0,16);
    addstr("community (if known):");
    escpress = query(data3, l, d, NEW, STR, 22, 16, 0, 16);
    if (strlen(data3) == 0) strcpy(data3, "public");
    sprintf(inputdata, "%s %s %s", data1, data3, data2);
    break;
  case 6:
    addstr("Enter address of component (ESC to cancel):");
    gotoxy(0,13);
    addstr("OR, enter IP name of component:");
    escpress = query(inputdata, l, d, ESCNEW, STR, 45, 12, 0, 16);
    if (escpress || strlen(inputdata) == 0) {
      m2choice = 0;
      strcpy(inputdata, "none ");
    }
    else strcat(inputdata, " \0");
    escpress = query(data1, l, d, NEW, STR, 32, 13, 0, 35);
    if (strlen(data1) == 0) {
      if (m2choice == 0)
	break;
      strcat(inputdata, "none ");
    }
    else
      strcat(inputdata, data1);
    m2choice = 6;
    gotoxy(0,15);
    addstr("Map (m) or confirm (c)?");
    escpress = query(data1, l, d, NEW, STR, 24, 15, 1, 1);
    if (data1[0] == 'm' || data1[0] == 'M') strcat(inputdata, " m");
    else strcat(inputdata, " c");
    gotoxy(0,17);
    addstr("Ignore non-snmp components (y/n)?");
    escpress = query(data1, l, d, NEW, YN, 34, 17, 1, 1);
    if (l) strcat(inputdata, " snmp");
    else   strcat(inputdata, " any");
    break;
  case 7:
    addstr("Enter address of starting node in network (ESC to cancel):");
    gotoxy(0,13);
    addstr("Or, enter name:");
    escpress = query(inputdata, l, d, ESCNEW, STR, 59, 12, 0, 16);
    if (escpress || strlen(inputdata) == 0) {
      m2choice = 0;
      if (escpress) break;
      strcpy(inputdata, "none ");
    }
    else strcat(inputdata, " \0");
    escpress = query(data1, l, d, NEW, STR, 16, 13, 0, 35);
    if (strlen(data1) == 0) {
      if (m2choice == 0)
	break;
      strcat(inputdata, "none ");
    }
    else {
      strcat(inputdata, data1);
      strcat(inputdata, " \0");
    }
    m2choice = 7;
    gotoxy(0,15);
    addstr("Enter address of node at end of route (ESC to cancel):");
    gotoxy(0,16);
    addstr("Or, enter name:");
    escpress = query(data1, l, d, ESCNEW, STR, 55, 15, 0, 16);
    if (escpress || strlen(data1) == 0) {
      m2choice = 0;
      if (escpress) break;
      strcat(inputdata, "none ");
    }
    else {
      strcat(inputdata, data1);
      strcat(inputdata, " \0");
    }
    escpress = query(data1, l, d, NEW, STR, 16, 16, 0, 35);
    if (strlen(data1) == 0) {
      if (m2choice == 0)
	break;
      strcat(inputdata, "none ");
    }
    else {
      strcat(inputdata, data1);
      strcat(inputdata, " \0");
    }
    m2choice = 7;
    gotoxy(0,17);
    addstr("Enter community name (if known): ");
    escpress = query(data1, l, d, NEW, STR, 33, 17, 0, 16);
    if(strlen(data1) == 0) strcat(inputdata, "public");
    else strcat(inputdata, data1);
    gotoxy(0,19);
    addstr("Add components on route to network (y/n)?");
    escpress = query(data1, l, d, NEW, YN, 42, 19, 0, 1);
    if (l) strcat(inputdata, " 1");
    else   strcat(inputdata, " 0");
    break;
  case 5:;
  default:
    addstr("Not yet implemented - Press a key to continue... ");
    refresh();
    c = getch();
    m2choice = 0;
    break;
  }
  DeleteMenu("menu2");
  return m2choice;
}

int iomgr::getmenu3choice(msgstr inputdata) {
  int m2choice;
  m2choice = menu3.getresponse(1,1);  /* leave on screen */
  inputdata[0] = '\0';
  DeleteMenu("menu3");
  return m2choice;
}

int iomgr::getnextinput(int& m2choice, msgstr inputdata) {
  int m1choice, escpress;
  long l;
  double d;
  mstring nextstr;
  m2choice = 0;
  inputdata[0] = '\0';
  InitTerm();
  m1choice = menu1.getresponse(0,1);
  switch (m1choice) {
    case 1:;
    case 2: 
      gotoxy(0,12);
      addstr("Enter name of file (ESC to cancel):");
      escpress = query(inputdata, l, d, ESCNEW, STR, 36, 12, 0, 30);
      if (escpress || strlen(inputdata) == 0)
	m1choice = 0;
      break;
    case 3:               /* print */
      m2choice = getmenu3choice(inputdata);
      if (m2choice == 0)
	m1choice = 0;
      break;
    case 4:               /* log file */
      if (logfileOnOff(inputdata) == 0)
	m1choice = 0;
      break;
    case 5:
      m2choice = 1;
      gotoxy(0,12);
      addstr("Enter IP address of new component (ESC to cancel):");
      escpress = query(inputdata, l, d, ESCNEW, STR, 51, 12, 0, 16);
      if (escpress) {
	m1choice = 0;
	break;
      }
      if (strlen(inputdata) == 0) {
	strcpy(inputdata, "none ");
	m2choice = 0;
      }
      else strcat(inputdata, " \0");
      gotoxy(0,14);
      addstr("Enter IP name of new component (if known):");
      escpress = query(nextstr, l, d, NEW, STR, 43, 14, 0, 35);
      if (strlen(nextstr) == 0) {
	if (m2choice == 0) {
	  m1choice = 0;
	  break;
	}
	strcat(inputdata, "none ");
      }
      else {
	strcat(inputdata, nextstr);
	strcat(inputdata, " \0");
      }
      gotoxy(0,16);
      addstr("Enter community of new component (if known):");
      escpress = query(nextstr, l, d, NEW, STR, 45, 16, 0, 16);
      if (strlen(nextstr) == 0) strcat(inputdata, "public");
      else strcat(inputdata, nextstr);
      break;
    case 6:
      m2choice = 0;
      gotoxy(0,12);
      addstr("Enter IP address of component to delete (ESC to cancel):");
      escpress = query(inputdata, l, d, ESCNEW, STR, 56, 12, 0, 16);
      if (escpress || strlen(inputdata) == 0) m1choice = 0;
      else strcat(inputdata, " none");
      break;
    case 7:
      m2choice = getmenu2choice(inputdata);
      if (m2choice == 0)
	m1choice = 0;
      break;
    case 8:
      break;
    default:
      break;
  }
  DeleteMenu("menu1");
  QuitTerm();
  return m1choice;
}

int iomgr::savefilefirst(mstring filename) {
  int escpress, i;
  char s[50];
  long l;
  double d;
  InitTerm();
  clear();
  addstr("\nWARNING--> You have made changes.\n");
  addstr("  Save current file before loading (Y/N)?");
  escpress = query(s, l, d, NEW, YN, 42, 2, 1, 1);
  if (l == 1) 
    if (strlen(filename) == 0)
      strcpy(filename, "default.save");
  i = l;
  QuitTerm();
  return i;
}

/*
 *  Print a line. 'Where' controls where msg goes.
 *  NOPRESS = No 'Press key'; PRESS = 'Press key'; LOG = logfile only; NOLOG
 */
void iomgr::printline(int where, mstring data) {
  char c;
  printf("%s\n", data);
  if (where == PRESS) {
    printf("    Press RETURN to continue... ");
    c = getchar();
  }
}

void drawbox(int x, int y, char c) {
  x = x - 3;
  y = y - 1;
  gotoxy(x,y);
  addstr("+-----+");
  gotoxy(x,y+1);
  addstr("|  ");
  addch(c);
  addstr("  |");
  gotoxy(x,y+2);
  addstr("+-----+");
}

void hconn(int x, int y) {
  x += 4;
  gotoxy(x,y);
  addstr("----");
}

void vconn(int x, int y) {
  y += 2;
  gotoxy(x,y);
  addch('|');
  gotoxy(x,y+1);
  addch('|');
}

void iomgr::titlescreen() {
  int x;
  char c;
  long t;
  InitTerm();
  standout();
  gotoxy(2,1);
  for (x = 0; x < 76; x++)
    addch(' ');
  refresh();
  for (x = 2; x < LINES-2; x++) {
    gotoxy(2,x);
    addstr("  ");
    gotoxy(76, x);
    addstr("  ");
    refresh();
  }
  gotoxy(2, LINES-2);
  for (x = 0; x < 76; x++)
    addch(' ');
  refresh();
  standend();
  gotoxy(23,10);
  addstr("--- NETWORK MANAGEMENT TOOLS ---");
  gotoxy(24,12);
  addstr("(with temporary text interface)");
  gotoxy(8, LINES-3);
  addstr("University of Washington");
  gotoxy(60, LINES-3);
  addstr("Walt Reissig");
  refresh();
  t = time(NULL) + 5;
  while (time(NULL) < t+3);
  drawbox(45,6,'W');
  gotoxy(78,LINES-2);
  refresh();
  while (time(NULL) < t+5);
  drawbox(34,6,'T');
  drawbox(56,6,'O');
  hconn(34,6);
  hconn(45,6);
  gotoxy(78,LINES-2);
  refresh();
  while (time(NULL) < t+7);
  drawbox(23,6,'E');
  drawbox(67,6,'R');
  hconn(23,6);
  hconn(56,6);
  gotoxy(78,LINES-2);
  refresh();
  while (time(NULL) < t+9);
  drawbox(12,6,'N');
  drawbox(67,11,'K');
  hconn(12,6);
  vconn(67,6);
  gotoxy(78,LINES-2);
  refresh();
  while (time(NULL) < t+13);
  drawbox(34,16,'A');
  gotoxy(78,LINES-2);
  refresh();
  while (time(NULL) < t+15);
  drawbox(23,16,'N');
  drawbox(45,16,'G');
  hconn(23,16);
  hconn(34,16);
  gotoxy(78,LINES-2);
  refresh();
  while (time(NULL) < t+17);
  drawbox(12,16,'A');
  drawbox(56,16,'E');
  hconn(12,16);
  hconn(45,16);
  gotoxy(78,LINES-2);
  refresh();
  while (time(NULL) < t+19);
  drawbox(12,11,'M');
  drawbox(67,16,'R');
  vconn(12,11);
  hconn(56,16);
  gotoxy(78,LINES-2);
  refresh();
  while (time(NULL) < t+23);
  vconn(12,6);
  vconn(67,11);
  gotoxy(78,LINES-2);
  refresh();
  c = getch();
  QuitTerm();
}

/*
 *  Return string user input.
 */ 
void iomgr::getresponse(mstring resp, int clr, int line, mstring prompt) {
  int xstart, escpress, length;
  long l;
  double d;
  if (clr) clear();
  xstart = strlen(prompt) + 1;
  gotoxy(0,line);
  length = 50;
  if ((80 - xstart) < length) length = 80 - xstart;
  escpress = query(resp, l, d, ESCNEW, STR, xstart, line, 0, length);
}

/*
 *  Return integer user input.
 */
void iomgr::getresponse(int& i, int clr, int line, mstring prompt) {
  int xstart, escpress, length;
  mstring resp;
  long l;
  double d;
  if (clr) clear();
  xstart = strlen(prompt) + 1;
  gotoxy(0,line);
  length = 6;
  if ((80 - xstart) < length) length = 80 - xstart;
  escpress = query(resp, l, d, ESCNEW, LINT, xstart, line, 0, length);
  i = int(l);
}

/*
 *  Return 'Y' or 'N' user input.
 */
void iomgr::getresponse(char& c, int clr, int line, mstring prompt) {
  int xstart, escpress;
  mstring resp;
  long l;
  double d;
  if (clr) clear();
  xstart = strlen(prompt) + 1;
  gotoxy(0,line);
  escpress = query(resp, l, d, ESCNEW, YN, xstart, line, 0, 1);
  if (l == 1) c = 'Y';
  else c = 'N';
}

void iomgr::CursesOn() {
  InitTerm();
}

void iomgr::CursesOff() {
  QuitTerm();
}

void iomgr::CursesClrScrn() {
  clear();
}

/*
 *  Print a line. 'Where' controls where msg goes.
 *  NOPRESS = No 'Press key'; PRESS = 'Press key'; LOG = logfile only; NOLOG
 */
void iomgr::CursesPrtLine(int where, mstring data, int line) {
  char c;
  if (where != LOG) {
    gotoxy(0,line);
    addstr(data);
    refresh();
    if (where == PRESS) {
      gotoxy(4, line+2);
      addstr("Press RETURN to continue... ");
      refresh();
      c = getch();
    }
  }
}

int iomgr::hasOutput(int& m1, int& m2, msgstr outdata) {
  int nrRead;
  if (nextToRead < nrOutWaiting) {
    msgMgr.decipherMsg(outbuf[nextToRead++], m1, m2, outdata);
    return 1;
  }
  nextToRead = nrOutWaiting = 0;
  nrRead = msgMgr.recvMsgBuf(outbuf);
  if (nrRead > 0) {
    msgMgr.decipherMsg(outbuf[nextToRead++], m1, m2, outdata);
    nrOutWaiting = nrRead;
    return 1;
  }
  return 0;
}

void iomgr::doOutputStuff(int m1, int m2, msgstr outdata) {
  switch (m1) {
  case 210:           /* dialog */
    switch (m2) {
    case 1: dialogBoxOpen = 1; break;
    case 2: dialogBoxOpen = 0; break;
    case 3: printline(NOPRESS, outdata); break;
    case 4: printline(PRESS, outdata);
            dialogBoxOpen = 0; break;
    }
  case 201: break;        /* draw node */
  case 202: break;        /* draw connection */
  case 203: break;        /* undraw node */
  case 204: break;        /* undraw connection */
  case 205: break;        /* modify node */
  case 206: break;        /* turn monitor on/off */
  default: break;
  }
}

void iomgr::ioLoop() {
  int menu1choice = 0, menu2choice = 0;
  msgstr inputstring;
  long t;
  for (;;) {
    if (!dialogBoxOpen) {
      menu1choice = getnextinput(menu2choice, inputstring);
      if (menu1choice != 0 || menu2choice != 0)
	while (msgMgr.sendOnelineMsg(menu1choice,menu2choice,inputstring) <=0);
      if (menu1choice == 8 && menu2choice == 0) {
	printf("quitting...\n");
	return;
      }
    }
    t = time(NULL) + 3;
    while (dialogBoxOpen || t > time(NULL)) {
      if (hasOutput(menu1choice, menu2choice, inputstring)) {
	doOutputStuff(menu1choice, menu2choice, inputstring);
	t = time(NULL) + 3;
      }
    }
  }
}

main() {
  iomgr iomanager;
/*  iomanager.titlescreen(); */
  iomanager.ioLoop();
  exit(10);
}
